@@ -2,11 +2,10 @@ use std::f64::INFINITY;
 
 use serde::{Deserialize, Serialize};
 
-use super::model_trait::{AsModel, SerializableModel};
-use super::ModelMessage;
+use super::model_trait::{DevsModel, Reportable, ReportableModel, SerializableModel};
+use super::{ModelMessage, ModelRecord};
 use crate::simulator::Services;
-use crate::utils::default_records_port_name;
-use crate::utils::error::SimulationError;
+use crate::utils::errors::SimulationError;
 
 use sim_derive::SerializableModel;
 
@@ -26,16 +25,12 @@ pub struct LoadBalancer {
 #[derive(Debug, Clone, Serialize, Deserialize)]
 struct PortsIn {
     job: String,
-    #[serde(default = "default_records_port_name")]
-    records: String,
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 struct PortsOut {
     flow_paths: Vec<String>,
-    #[serde(default = "default_records_port_name")]
-    records: String,
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
@@ -44,15 +39,15 @@ struct State {
     phase: Phase,
     until_next_event: f64,
     next_port_out: usize,
-    jobs: Vec<Job>,
-    records: Vec<Job>,
+    jobs: Vec<String>,
+    records: Vec<ModelRecord>,
 }
 
 impl Default for State {
     fn default() -> Self {
         Self {
-            phase: Phase::LoadBalancing,
-            until_next_event: 0.0,
+            phase: Phase::Passive,
+            until_next_event: INFINITY,
             next_port_out: 0,
             jobs: Vec::new(),
             records: Vec::new(),
@@ -62,52 +57,22 @@ impl Default for State {
 
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
 enum Phase {
+    Passive,
     LoadBalancing,
-    RecordsFetch,
-}
-
-#[derive(Debug, Clone, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
-struct Job {
-    content: String,
-    time: f64,
-    port_out: String,
 }
 
 impl LoadBalancer {
     pub fn new(job_port: String, flow_path_ports: Vec<String>, store_records: bool) -> Self {
         Self {
-            ports_in: PortsIn {
-                job: job_port,
-                records: default_records_port_name(),
-            },
+            ports_in: PortsIn { job: job_port },
             ports_out: PortsOut {
                 flow_paths: flow_path_ports,
-                records: default_records_port_name(),
             },
             store_records,
-            state: Default::default(),
+            state: State::default(),
         }
     }
 
-    fn request_records(
-        &mut self,
-        _incoming_message: &ModelMessage,
-        _services: &mut Services,
-    ) -> Result<(), SimulationError> {
-        self.state.phase = Phase::RecordsFetch;
-        self.state.until_next_event = 0.0;
-        Ok(())
-    }
-
-    fn ignore_request(
-        &mut self,
-        _incoming_message: &ModelMessage,
-        _services: &mut Services,
-    ) -> Result<(), SimulationError> {
-        Ok(())
-    }
-
     fn pass_job(
         &mut self,
         incoming_message: &ModelMessage,
@@ -115,96 +80,67 @@ impl LoadBalancer {
     ) -> Result<(), SimulationError> {
         self.state.phase = Phase::LoadBalancing;
         self.state.until_next_event = 0.0;
-        self.state.jobs.push(Job {
-            content: incoming_message.content.clone(),
-            time: services.global_time(),
-            port_out: self.ports_out.flow_paths[self.state.next_port_out].clone(),
-        });
-        self.state.next_port_out = (self.state.next_port_out + 1) % self.ports_out.flow_paths.len();
-        Ok(())
-    }
-
-    fn save_job(
-        &mut self,
-        incoming_message: &ModelMessage,
-        services: &mut Services,
-    ) -> Result<(), SimulationError> {
-        self.state.phase = Phase::LoadBalancing;
-        self.state.until_next_event = 0.0;
-        self.state.jobs.push(Job {
-            content: incoming_message.content.clone(),
-            time: services.global_time(),
-            port_out: self.ports_out.flow_paths[self.state.next_port_out].clone(),
-        });
-        self.state.records.push(Job {
-            content: incoming_message.content.clone(),
-            time: services.global_time(),
-            port_out: self.ports_out.flow_paths[self.state.next_port_out].clone(),
-        });
-        self.state.next_port_out = (self.state.next_port_out + 1) % self.ports_out.flow_paths.len();
+        self.state.jobs.push(incoming_message.content.clone());
+        self.record(
+            services.global_time(),
+            String::from("Arrival"),
+            incoming_message.content.clone(),
+        );
         Ok(())
     }
 
     fn passivate(&mut self) -> Result<Vec<ModelMessage>, SimulationError> {
-        self.state.phase = Phase::LoadBalancing;
+        self.state.phase = Phase::Passive;
         self.state.until_next_event = INFINITY;
         Ok(Vec::new())
     }
 
-    fn send_job(&mut self) -> Result<Vec<ModelMessage>, SimulationError> {
+    fn send_job(&mut self, services: &mut Services) -> Result<Vec<ModelMessage>, SimulationError> {
         self.state.until_next_event = 0.0;
-        let job = self.state.jobs.remove(0);
+        self.state.next_port_out = (self.state.next_port_out + 1) % self.ports_out.flow_paths.len();
+        self.record(
+            services.global_time(),
+            String::from("Departure"),
+            format![
+                "{} on {}",
+                self.state.jobs[0].clone(),
+                self.ports_out.flow_paths[self.state.next_port_out].clone()
+            ],
+        );
         Ok(vec![ModelMessage {
-            port_name: job.port_out,
-            content: job.content,
+            port_name: self.ports_out.flow_paths[self.state.next_port_out].clone(),
+            content: self.state.jobs.remove(0),
         }])
     }
 
-    fn release_records(&mut self) -> Result<Vec<ModelMessage>, SimulationError> {
-        self.state.phase = Phase::LoadBalancing;
-        self.state.until_next_event = 0.0;
-        Ok(vec![ModelMessage {
-            port_name: self.ports_out.records.clone(),
-            content: serde_json::to_string(&self.state.records).unwrap(),
-        }])
+    fn record(&mut self, time: f64, action: String, subject: String) {
+        if self.store_records {
+            self.state.records.push(ModelRecord {
+                time,
+                action,
+                subject,
+            })
+        }
     }
 }
 
-impl AsModel for LoadBalancer {
-    fn status(&self) -> String {
-        format!["Listening for {}s", self.ports_in.job]
-    }
-
+impl DevsModel for LoadBalancer {
     fn events_ext(
         &mut self,
         incoming_message: &ModelMessage,
         services: &mut Services,
     ) -> Result<(), SimulationError> {
-        if incoming_message.port_name == self.ports_in.records && self.store_records {
-            self.request_records(incoming_message, services)
-        } else if incoming_message.port_name == self.ports_in.records && !self.store_records {
-            self.ignore_request(incoming_message, services)
-        } else if incoming_message.port_name == self.ports_in.job && self.store_records {
-            self.save_job(incoming_message, services)
-        } else if incoming_message.port_name == self.ports_in.job && !self.store_records {
-            self.pass_job(incoming_message, services)
-        } else {
-            Err(SimulationError::InvalidModelState)
-        }
+        self.pass_job(incoming_message, services)
     }
 
     fn events_int(
         &mut self,
-        _services: &mut Services,
+        services: &mut Services,
     ) -> Result<Vec<ModelMessage>, SimulationError> {
-        if self.state.phase == Phase::RecordsFetch {
-            self.release_records()
-        } else if self.state.phase == Phase::LoadBalancing && self.state.jobs.is_empty() {
+        if self.state.jobs.is_empty() {
             self.passivate()
-        } else if self.state.phase == Phase::LoadBalancing && !self.state.jobs.is_empty() {
-            self.send_job()
         } else {
-            Err(SimulationError::InvalidModelState)
+            self.send_job(services)
         }
     }
 
@@ -216,3 +152,15 @@ impl AsModel for LoadBalancer {
         self.state.until_next_event
     }
 }
+
+impl Reportable for LoadBalancer {
+    fn status(&self) -> String {
+        format!["Listening for {}s", self.ports_in.job]
+    }
+
+    fn records(&self) -> &Vec<ModelRecord> {
+        &self.state.records
+    }
+}
+
+impl ReportableModel for LoadBalancer {}
