@@ -2,11 +2,10 @@ use std::f64::INFINITY;
 
 use serde::{Deserialize, Serialize};
 
-use super::model_trait::{AsModel, SerializableModel};
-use super::ModelMessage;
+use super::model_trait::{DevsModel, Reportable, ReportableModel, SerializableModel};
+use super::{ModelMessage, ModelRecord};
 use crate::simulator::Services;
-use crate::utils::default_records_port_name;
-use crate::utils::error::SimulationError;
+use crate::utils::errors::SimulationError;
 
 use sim_derive::SerializableModel;
 
@@ -27,15 +26,18 @@ pub struct Storage {
 struct PortsIn {
     put: String,
     get: String,
-    #[serde(default = "default_records_port_name")]
-    records: String,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+enum ArrivalPort {
+    Put,
+    Get,
+    Unknown,
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
 struct PortsOut {
     stored: String,
-    #[serde(default = "default_records_port_name")]
-    records: String,
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
@@ -44,7 +46,7 @@ struct State {
     phase: Phase,
     until_next_event: f64,
     job: Option<String>,
-    records: Vec<Job>,
+    records: Vec<ModelRecord>,
 }
 
 impl Default for State {
@@ -62,21 +64,6 @@ impl Default for State {
 enum Phase {
     Passive,
     JobFetch,
-    RecordsFetch,
-}
-
-#[derive(Debug, Clone, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
-pub struct Job {
-    pub operation: Operation,
-    pub content: Option<String>,
-    pub time: f64,
-}
-
-#[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
-pub enum Operation {
-    Get,
-    Put,
 }
 
 impl Storage {
@@ -90,92 +77,56 @@ impl Storage {
             ports_in: PortsIn {
                 put: put_port,
                 get: get_port,
-                records: default_records_port_name(),
             },
             ports_out: PortsOut {
                 stored: stored_port,
-                records: default_records_port_name(),
             },
             store_records,
-            state: Default::default(),
+            state: State::default(),
         }
     }
 
-    fn request_records(
-        &mut self,
-        _incoming_message: &ModelMessage,
-        _services: &mut Services,
-    ) -> Result<(), SimulationError> {
-        self.state.phase = Phase::RecordsFetch;
-        self.state.until_next_event = 0.0;
-        Ok(())
+    fn arrival_port(&self, message_port: &str) -> ArrivalPort {
+        if message_port == self.ports_in.put {
+            ArrivalPort::Put
+        } else if message_port == self.ports_in.get {
+            ArrivalPort::Get
+        } else {
+            ArrivalPort::Unknown
+        }
     }
 
-    fn ignore_request(
-        &mut self,
-        _incoming_message: &ModelMessage,
-        _services: &mut Services,
-    ) -> Result<(), SimulationError> {
-        Ok(())
-    }
-
-    fn get_stored_value(&mut self) -> Result<(), SimulationError> {
+    fn get_job(&mut self) -> Result<(), SimulationError> {
         self.state.phase = Phase::JobFetch;
         self.state.until_next_event = 0.0;
         Ok(())
     }
 
-    fn save_job(
+    fn hold_job(
         &mut self,
         incoming_message: &ModelMessage,
         services: &mut Services,
     ) -> Result<(), SimulationError> {
         self.state.job = Some(incoming_message.content.clone());
-        self.state.records.push(Job {
-            operation: Operation::Put,
-            content: Some(incoming_message.content.clone()),
-            time: services.global_time(),
-        });
+        self.record(
+            services.global_time(),
+            String::from("Arrival"),
+            incoming_message.content.clone(),
+        );
         Ok(())
     }
 
-    fn hold_job(&mut self, incoming_message: &ModelMessage) -> Result<(), SimulationError> {
-        self.state.job = Some(incoming_message.content.clone());
-        Ok(())
-    }
-
-    fn release_records(&mut self) -> Result<Vec<ModelMessage>, SimulationError> {
-        self.state.phase = Phase::Passive;
-        self.state.until_next_event = INFINITY;
-        Ok(vec![ModelMessage {
-            port_name: self.ports_out.records.clone(),
-            content: serde_json::to_string(&self.state.records).unwrap(),
-        }])
-    }
-
-    fn release_job(&mut self) -> Result<Vec<ModelMessage>, SimulationError> {
-        self.state.phase = Phase::Passive;
-        self.state.until_next_event = INFINITY;
-        match &self.state.job {
-            Some(job) => Ok(vec![ModelMessage {
-                port_name: self.ports_out.stored.clone(),
-                content: job.clone(),
-            }]),
-            None => Ok(Vec::new()),
-        }
-    }
-
-    fn save_and_release_job(
+    fn release_job(
         &mut self,
         services: &mut Services,
     ) -> Result<Vec<ModelMessage>, SimulationError> {
         self.state.phase = Phase::Passive;
         self.state.until_next_event = INFINITY;
-        self.state.records.push(Job {
-            operation: Operation::Get,
-            content: self.state.job.clone(),
-            time: services.global_time(),
-        });
+        self.record(
+            services.global_time(),
+            String::from("Departure"),
+            self.state.job.clone().unwrap_or_else(|| "None".to_string()),
+        );
         match &self.state.job {
             Some(job) => Ok(vec![ModelMessage {
                 port_name: self.ports_out.stored.clone(),
@@ -190,33 +141,28 @@ impl Storage {
         self.state.until_next_event = INFINITY;
         Ok(Vec::new())
     }
-}
 
-impl AsModel for Storage {
-    fn status(&self) -> String {
-        match &self.state.job {
-            Some(stored) => format!["Storing {}", stored],
-            None => String::from("Empty"),
+    fn record(&mut self, time: f64, action: String, subject: String) {
+        if self.store_records {
+            self.state.records.push(ModelRecord {
+                time,
+                action,
+                subject,
+            })
         }
     }
+}
 
+impl DevsModel for Storage {
     fn events_ext(
         &mut self,
         incoming_message: &ModelMessage,
         services: &mut Services,
     ) -> Result<(), SimulationError> {
-        if incoming_message.port_name == self.ports_in.records && self.store_records {
-            self.request_records(incoming_message, services)
-        } else if incoming_message.port_name == self.ports_in.records && !self.store_records {
-            self.ignore_request(incoming_message, services)
-        } else if incoming_message.port_name == self.ports_in.put && self.store_records {
-            self.save_job(incoming_message, services)
-        } else if incoming_message.port_name == self.ports_in.put && !self.store_records {
-            self.hold_job(incoming_message)
-        } else if incoming_message.port_name == self.ports_in.get {
-            self.get_stored_value()
-        } else {
-            Err(SimulationError::InvalidModelState)
+        match self.arrival_port(&incoming_message.port_name) {
+            ArrivalPort::Put => self.hold_job(incoming_message, services),
+            ArrivalPort::Get => self.get_job(),
+            ArrivalPort::Unknown => Err(SimulationError::InvalidMessage),
         }
     }
 
@@ -224,16 +170,9 @@ impl AsModel for Storage {
         &mut self,
         services: &mut Services,
     ) -> Result<Vec<ModelMessage>, SimulationError> {
-        if self.state.phase == Phase::RecordsFetch {
-            self.release_records()
-        } else if self.state.phase == Phase::Passive {
-            self.passivate()
-        } else if self.state.phase == Phase::JobFetch && self.store_records {
-            self.save_and_release_job(services)
-        } else if self.state.phase == Phase::JobFetch && !self.store_records {
-            self.release_job()
-        } else {
-            Err(SimulationError::InvalidModelState)
+        match &self.state.phase {
+            Phase::Passive => self.passivate(),
+            Phase::JobFetch => self.release_job(services),
         }
     }
 
@@ -245,3 +184,18 @@ impl AsModel for Storage {
         self.state.until_next_event
     }
 }
+
+impl Reportable for Storage {
+    fn status(&self) -> String {
+        match &self.state.job {
+            Some(stored) => format!["Storing {}", stored],
+            None => String::from("Empty"),
+        }
+    }
+
+    fn records(&self) -> &Vec<ModelRecord> {
+        &self.state.records
+    }
+}
+
+impl ReportableModel for Storage {}
