@@ -18,8 +18,8 @@ use std::f64::INFINITY;
 
 use serde::{Deserialize, Serialize};
 
-use crate::models::{AsModel, Model, ModelMessage};
-use crate::utils::error::SimulationError;
+use crate::models::{DevsModel, Model, ModelMessage, ModelRecord, Reportable};
+use crate::utils::errors::SimulationError;
 use crate::utils::set_panic_hook;
 
 pub mod coupling;
@@ -28,7 +28,7 @@ pub mod web;
 
 pub use self::coupling::{Connector, Message};
 pub use self::services::Services;
-pub use self::web::WebSimulation;
+pub use self::web::Simulation as WebSimulation;
 
 /// The `Simulation` struct is the core of sim, and includes everything
 /// needed to run a simulation - models, connectors, and a random number
@@ -80,7 +80,7 @@ impl Simulation {
     /// This method provides a mechanism for getting the status of any model
     /// in a simulation.  The method takes the model ID as an argument, and
     /// returns the current status string for that model.
-    pub fn status(&self, model_id: &str) -> Result<String, SimulationError> {
+    pub fn get_status(&self, model_id: &str) -> Result<String, SimulationError> {
         Ok(self
             .models
             .iter()
@@ -89,6 +89,18 @@ impl Simulation {
             .status())
     }
 
+    /// This method provides a mechanism for getting the records of any model
+    /// in a simulation.  The method takes the model ID as an argument, and
+    /// returns the records for that model.
+    pub fn get_records(&self, model_id: &str) -> Result<&Vec<ModelRecord>, SimulationError> {
+        Ok(self
+            .models
+            .iter()
+            .find(|model| model.id() == model_id)
+            .ok_or(SimulationError::ModelNotFound)?
+            .records())
+    }
+
     /// To enable simulation replications, the reset method resets the state
     /// of the simulation, except for the random number generator.
     /// Recreating a simulation from scratch for additional replications
@@ -234,7 +246,7 @@ impl Simulation {
             .collect();
         errors?;
         self.messages = next_messages;
-        Ok(self.get_messages().to_vec())
+        Ok(self.get_messages().clone())
     }
 
     /// This method executes simulation `step` calls, until a global time
@@ -264,7 +276,7 @@ impl Simulation {
                 message_records.extend(self.messages.clone());
                 Ok(Vec::new())
             })
-            .find(|result| result.is_err())
+            .find(Result::is_err)
             .unwrap_or(Ok(message_records))
     }
 }
